/*
 * 11:35:13 21/06/99
 *
 * iOb.java - An Image Obfuscator for Web Pages
 * Copyright (C) 1998-1999 Romain Guy
 * powerteam@chez.com
 * www.chez.com/powerteam
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import java.net.URL;
import java.awt.Image;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.applet.Applet;
import java.awt.MediaTracker;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.net.MalformedURLException;
import java.awt.event.MouseMotionAdapter;

/**
 * iOb is an image obfuscator. This can be very useful if you
 * don't want your home page's visitors to be able to rip your
 * graphics. Indeed, right-clicking iOb won't ever provide the
 * 'Save Picture As' menu item.
 * @author Romain Guy
 * @version 1.2
 */

public class iOb extends Applet implements Runnable
{
  // ---- Some useful stuffs ----

  private Thread runner;
  private MediaTracker tracker;
  private Image obfuscatedImage;
  private boolean waitForFullLoading;
  private String statusInfo, link, target;

  /**
   * In the <code>init()</code> method, we simply add a
   * <code>MouseMotionListener</code>, create a
   * <code>MediaTracker</code> and then get the parameters.
   */

  public void init()
  {
    Color backColor = Color.white;

    // ---- We add a MouseMotionListener to display infos in the satus bar ----

    addMouseMotionListener(new MouseMotionAdapter()
    {
      public void mouseMoved(MouseEvent evt)
      {
        getAppletContext().showStatus(statusInfo);
      }
    });

    String data;
    tracker = new MediaTracker(this);

    // ---- Background color ----

    data = getParameter("bgcolor");
    if (data != null)
    {
      try
      {
        backColor = Color.decode(data);
      } catch (NumberFormatException nfe) {
      } finally {
        setBackground(backColor);
      }
    }
    data = null;

    // ---- We get the infos to be displayed in the status bar----

    statusInfo = getParameter("infos");
    if (statusInfo == null)
      statusInfo = "iOb - Image Obfuscator - (C)1999 Romain Guy - www.chez.com/powerteam";

    // ---- Should we wait for entire image loading before displaying it ? ----

    data = getParameter("wait");
    if (data != null)
      waitForFullLoading = data.equalsIgnoreCase("yes");
    data = null;

    // ---- Yeah ! You can even link to another document ----

    link = getParameter("link");
    if (link != null)
    {
      setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
      target = getParameter("target");
      if (target == null)
        target = "_self";
      addMouseListener(new MouseAdapter()
      {
        public void mouseReleased(MouseEvent evt)
        {
          try
          {
            getAppletContext().showDocument(new URL(getDocumentBase(), link), target);
          } catch (MalformedURLException mue) { }
        }
      });
    }

    // ---- Finally, we get the image ----

    data = getParameter("image");
    if (data == null)
    {
      statusInfo = "iOb: missing parameter: image URL";
      stop();
    } else {
      obfuscatedImage = getImage(getDocumentBase(), data);
      tracker.addImage(obfuscatedImage, 0);
    }
  }

  /**
   * This simply starts the main iOb's Thread.
   */

  public void start()
  {
    if (runner == null)
    {
      runner = new Thread(this);
      runner.start();
    }
  }

  /**
   * Here we kill the Thread.
   */

  public void stop()
  {
    runner = null;
  }

  /**
   * In this method, we repaint until the picture has been
   * fully loaded.
   */

  public void run()
  {
    if (waitForFullLoading)
    {
      try
      {
        tracker.waitForID(0);
      } catch (InterruptedException e) { return; }
    }
    Thread me = Thread.currentThread();
    while (runner == me && tracker.statusID(0, false) != MediaTracker.COMPLETE && !tracker.isErrorID(0))
      repaint();
  }

  public void destroy()
  {
    obfuscatedImage.flush();
    obfuscatedImage = null;
  }

  public void update(Graphics g)
  {
    paint(g);
  }

  public void paint(Graphics g)
  {
    if (tracker.isErrorID(0))
    {
      g.setColor(Color.white);
      g.fillRect(10, 10, 20, 20);
      g.setColor(Color.black);
      g.drawRect(9, 9, 21, 21);
      g.setColor(Color.red);
      g.drawString("Error loading image", 35, 24);
      g.setFont(new java.awt.Font("monospaced", java.awt.Font.BOLD, 21));
      g.drawString("X", 13, 27);
      return;
    }
    g.drawImage(obfuscatedImage, 0, 0, this);
  }

  public String getAppletInfo()
  {
    return("iOb (C)1999 Romain Guy\npowerteam@chez.com\nwww.chez.com/powerteam");
  }
}

// End of iOb.java
